@@ -1,5 +1,5 @@
 /* Tests of fstat() function.
-   Copyright (C) 2011-2023 Free Software Foundation, Inc.
+   Copyright (C) 2011-2025 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@ SIGNATURE_CHECK (fstat, int, (int, struct stat *));
 #include "macros.h"
 
 int
-main (int argc, char *argv[])
+main ()
 {
   /* Test behaviour for invalid file descriptors.  */
   {
@@ -46,5 +46,5 @@ main (int argc, char *argv[])
     ASSERT (errno == EBADF);
   }
 
-  return 0;
+  return test_exit_status;
 }
