@@ -1,9 +1,9 @@
 /* Test intprops.h.
-   Copyright (C) 2011-2020 Free Software Foundation, Inc.
+   Copyright (C) 2011-2025 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3 of the License, or
+   the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -16,9 +16,12 @@
 
 /* Written by Paul Eggert.  */
 
+#include <config.h>
+
+#if _GL_GNUC_PREREQ (4, 3)
+
 /* Tell gcc not to warn about the long expressions that the overflow
    macros expand to, or about the (X < 0) expressions.  */
-#if 4 < __GNUC__ + (3 <= __GNUC_MINOR__)
 # pragma GCC diagnostic ignored "-Woverlength-strings"
 # pragma GCC diagnostic ignored "-Wtype-limits"
 
@@ -28,12 +31,12 @@
 
 #endif
 
-#include <config.h>
+#ifdef TEST_STDCKDINT
+# include <stdckdint.h>
+#else
+# include "intprops.h"
+#endif
 
-#include "intprops.h"
-#include "verify.h"
-
-#include <stdbool.h>
 #include <inttypes.h>
 #include <limits.h>
 
@@ -41,14 +44,17 @@
 
 /* Compile-time verification of expression X.
    In this file, we need it as a statement, rather than as a declaration.  */
-#define verify_stmt(x) do { verify (x); } while (0)
+#define verify_stmt(x) do { static_assert (x); } while (0)
 
 /* VERIFY (X) uses a static assertion for compilers that are known to work,
    and falls back on a dynamic assertion for other compilers.
+   But it ignores X if testing stdckdint.h.
    These tests should be checkable via 'verify' rather than 'ASSERT', but
    using 'verify' would run into a bug with HP-UX 11.23 cc; see
    <https://lists.gnu.org/r/bug-gnulib/2011-05/msg00401.html>.  */
-#if __GNUC__ || __clang__ || __SUNPRO_C
+#ifdef TEST_STDCKDINT
+# define VERIFY(x) ((void) 0)
+#elif __GNUC__ || __clang__ || __SUNPRO_C
 # define VERIFY(x) verify_stmt (x)
 #else
 # define VERIFY(x) ASSERT (x)
@@ -56,6 +62,21 @@
 
 #define DONTCARE __LINE__
 
+#ifndef TEST_STDCKDINT
+/* Check that INT_PROMOTE promotes to int.
+   GCC < 4.9 lacks _Generic even though it may claim C11 conformance.  */
+# if (201112 <= __STDC_VERSION__ \
+      && (!defined __GNUC__ || 4 < __GNUC__ + (9 <= __GNUC_MINOR__) \
+          || defined __clang__))
+int check_INT_PROMOTE = _Generic (INT_PROMOTE ((short int) 0), int: 0);
+# endif
+/* For other compilers, check the size and sign of INT_PROMOTE (x).  */
+int check_INT_PROMOTE_size
+    [2 * (sizeof (INT_PROMOTE ((short int) 0)) == sizeof (int)) - 1];
+int check_INT_PROMOTE_sign
+    [2 * (INT_PROMOTE ((short int) -1) < 0) - 1];
+#endif
+
 int int_minus_2 = -2;
 int int_1 = 1;
 
@@ -65,6 +86,7 @@ main (void)
   /* Use VERIFY for tests that must be integer constant expressions,
      ASSERT otherwise.  */
 
+#ifndef TEST_STDCKDINT
   /* TYPE_IS_INTEGER.  */
   ASSERT (TYPE_IS_INTEGER (bool));
   ASSERT (TYPE_IS_INTEGER (char));
@@ -98,6 +120,9 @@ main (void)
   ASSERT (TYPE_SIGNED (double));
   ASSERT (TYPE_SIGNED (long double));
 
+  /* Check that INT_PROMOTE is a no-op on floats.  */
+  ASSERT (INT_PROMOTE (2.71828) > 2);
+
   /* Integer representation.  Check that it is two's complement.  */
   VERIFY (INT_MIN + INT_MAX < 0);
 
@@ -161,12 +186,13 @@ main (void)
   VERIFY (INT_STRLEN_BOUND (int64_t) == sizeof ("-9223372036854775808") - 1);
   VERIFY (INT_BUFSIZE_BOUND (int64_t) == sizeof ("-9223372036854775808"));
   #endif
+#endif
 
   /* All the INT_<op>_RANGE_OVERFLOW tests are equally valid as
      INT_<op>_OVERFLOW tests, so define macros to do both.  OP is the
      operation, OPNAME its symbolic name, A and B its operands, T the
      result type, V the overflow flag, and VRES the result if V and if
-     two's complement.  CHECK_BINOP is for most binary operatinos,
+     two's complement.  CHECK_BINOP is for most binary operations,
      CHECK_SBINOP for binary +, -, * when the result type is signed,
      and CHECK_UNOP for unary operations.  */
   #define CHECK_BINOP(op, opname, a, b, t, v, vres)                       \
@@ -437,5 +463,5 @@ main (void)
   CHECK_REMAINDER (37*39u - 1, -39, true);
   CHECK_REMAINDER (LONG_MAX, -INT_MAX, false);
 
-  return 0;
+  return test_exit_status;
 }
