@@ -1,9 +1,9 @@
 /* provide a chdir function that tries not to fail due to ENAMETOOLONG
-   Copyright (C) 2004-2005, 2009-2020 Free Software Foundation, Inc.
+   Copyright (C) 2004-2005, 2009-2025 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3 of the License, or
+   the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -21,6 +21,11 @@
 
 #include "pathmax.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+
 /* On systems without PATH_MAX, presume that chdir accepts
    arbitrarily long directory names.  */
 #ifndef PATH_MAX
@@ -28,3 +33,8 @@
 #else
 int chdir_long (char *dir);
 #endif
+
+
+#ifdef __cplusplus
+}
+#endif
