@@ -1,5 +1,5 @@
 /* pcresearch.c - searching subroutines using PCRE for grep.
-   Copyright 2000, 2007, 2009-2023 Free Software Foundation, Inc.
+   Copyright 2000, 2007, 2009-2025 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,13 +12,11 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA
-   02110-1301, USA.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 #include <config.h>
 
-#include "search.h"
+#include <search.h>
 #include "die.h"
 
 #include <stdckdint.h>
@@ -39,6 +37,15 @@
 # define PCRE2_EXTRA_ASCII_BSD 0
 #endif
 
+/* Use PCRE2_MATCH_INVALID_UTF if supported and not buggy;
+   see <https://github.com/PCRE2Project/pcre2/issues/224>.
+   Assume the bug will be fixed after PCRE2 10.42.  */
+#if defined PCRE2_MATCH_INVALID_UTF && 10 < PCRE2_MAJOR + (42 < PCRE2_MINOR)
+enum { MATCH_INVALID_UTF = PCRE2_MATCH_INVALID_UTF };
+#else
+enum { MATCH_INVALID_UTF = 0 };
+#endif
+
 struct pcre_comp
 {
   /* General context for PCRE operations.  */
@@ -74,6 +81,15 @@ private_free (void *ptr, _GL_UNUSED void *unused)
   free (ptr);
 }
 
+void
+Pprint_version (void)
+{
+  char *buf = ximalloc (pcre2_config (PCRE2_CONFIG_VERSION, nullptr));
+  pcre2_config (PCRE2_CONFIG_VERSION, buf);
+  printf (_("\ngrep -P uses PCRE2 %s\n"), buf);
+  free (buf);
+}
+
 /* Match the already-compiled PCRE pattern against the data in SUBJECT,
    of size SEARCH_BYTES and starting with offset SEARCH_OFFSET, with
    options OPTIONS.
@@ -104,7 +120,7 @@ jit_exec (struct pcre_comp *pc, char const *subject, idx_t search_bytes,
             xalloc_die ();
           if (!pc->mcontext)
             pc->mcontext = pcre2_match_context_create (pc->gcontext);
-          pcre2_jit_stack_assign (pc->mcontext, NULL, pc->jit_stack);
+          pcre2_jit_stack_assign (pc->mcontext, nullptr, pc->jit_stack);
         }
       else if (e == PCRE2_ERROR_DEPTHLIMIT)
         {
@@ -121,116 +137,26 @@ jit_exec (struct pcre_comp *pc, char const *subject, idx_t search_bytes,
     }
 }
 
-/* Return true if E is an error code for bad UTF-8, and if pcre2_match
-   could return E because PCRE lacks PCRE2_MATCH_INVALID_UTF.  */
+/* Return true if E is an error code for bad UTF-8.  */
 static bool
 bad_utf8_from_pcre2 (int e)
 {
-#ifdef PCRE2_MATCH_INVALID_UTF
-  return false;
-#else
   return PCRE2_ERROR_UTF8_ERR21 <= e && e <= PCRE2_ERROR_UTF8_ERR1;
-#endif
 }
 
-#if ! PCRE2_EXTRA_ASCII_BSD
-/* Replace each \d in *KEYS_P with [0-9], to ensure that \d matches only ASCII
-   digits.  Now that we enable PCRE2_UCP for pcre regexps, \d would otherwise
-   match non-ASCII digits in some locales.  Use \p{Nd} if you require to match
-   those.  Similarly, replace each \D with [^0-9].
-   FIXME: remove in 2025, or whenever we no longer accommodate pcre2-10.42
-   and prior.  */
-static void
-pcre_pattern_expand_backslash_d (char **keys_p, idx_t *len_p)
-{
-  idx_t len = *len_p;
-  char *keys = *keys_p;
-  mbstate_t mb_state = { 0 };
-  char *new_keys = xnmalloc (len / 2 + 1, 5);
-  char *p = new_keys;
-  bool prev_backslash = false;
-
-  for (ptrdiff_t n; len; keys += n, len -= n)
-    {
-      n = mb_clen (keys, len, &mb_state);
-      switch (n)
-        {
-        case -2:
-          n = len;
-          FALLTHROUGH;
-        default:
-          if (prev_backslash)
-            {
-              prev_backslash = false;
-              *p++ = '\\';
-            }
-          p = mempcpy (p, keys, n);
-          break;
-
-        case -1:
-          if (prev_backslash)
-            {
-              prev_backslash = false;
-              *p++ = '\\';
-            }
-          memset (&mb_state, 0, sizeof mb_state);
-          n = 1;
-          FALLTHROUGH;
-        case 1:
-          if (prev_backslash)
-            {
-              prev_backslash = false;
-              switch (*keys)
-                {
-                case 'd':
-                  p = mempcpy (p, "[0-9]", 5);
-                  break;
-                case 'D':
-                  p = mempcpy (p, "[^0-9]", 6);
-                  break;
-                default:
-                  *p++ = '\\';
-                  *p++ = *keys;
-                  break;
-                }
-            }
-          else
-            {
-              if (*keys == '\\')
-                prev_backslash = true;
-              else
-                *p++ = *keys;
-            }
-          break;
-        }
-    }
-
-  if (prev_backslash)
-    *p++ = '\\';
-  *p = '\n';
-  free (*keys_p);
-  *keys_p = new_keys;
-  *len_p = p - new_keys;
-}
-#endif
-
 /* Compile the -P style PATTERN, containing SIZE bytes that are
    followed by '\n'.  Return a description of the compiled pattern.  */
 
 void *
 Pcompile (char *pattern, idx_t size, reg_syntax_t ignored, bool exact)
 {
-#if ! PCRE2_EXTRA_ASCII_BSD
-  pcre_pattern_expand_backslash_d (&pattern, &size);
-#endif
-
   PCRE2_SIZE e;
   int ec;
   int flags = PCRE2_DOLLAR_ENDONLY | (match_icase ? PCRE2_CASELESS : 0);
   char *patlim = pattern + size;
   struct pcre_comp *pc = ximalloc (sizeof *pc);
   pcre2_general_context *gcontext = pc->gcontext
-    = pcre2_general_context_create (private_malloc, private_free, NULL);
+    = pcre2_general_context_create (private_malloc, private_free, nullptr);
   pcre2_compile_context *ccontext = pcre2_compile_context_create (gcontext);
 
   if (localeinfo.multibyte)
@@ -241,14 +167,23 @@ Pcompile (char *pattern, idx_t size, reg_syntax_t ignored, bool exact)
              _("-P supports only unibyte locales on this platform"));
       if (! localeinfo.using_utf8)
         die (EXIT_TROUBLE, 0, _("-P supports only unibyte and UTF-8 locales"));
-      flags |= (PCRE2_UTF | PCRE2_UCP);
+
+      flags |= PCRE2_UTF;
+
+      /* If supported, consider invalid UTF-8 as a barrier not an error.  */
+      flags |= MATCH_INVALID_UTF;
+
+      /* If PCRE2_EXTRA_ASCII_BSD is available, use PCRE2_UCP
+         so that \d does not have the undesirable effect of matching
+         non-ASCII digits.  Otherwise (i.e., with PCRE2 10.42 and earlier),
+         escapes like \w have only their ASCII interpretations,
+         but that's better than the confusion that would ensue if \d
+         matched non-ASCII digits.  */
+      flags |= PCRE2_EXTRA_ASCII_BSD ? PCRE2_UCP : 0;
+
 #if 0
       /* Do not match individual code units but only UTF-8.  */
       flags |= PCRE2_NEVER_BACKSLASH_C;
-#endif
-#ifdef PCRE2_MATCH_INVALID_UTF
-      /* Consider invalid UTF-8 as a barrier, instead of error.  */
-      flags |= PCRE2_MATCH_INVALID_UTF;
 #endif
     }
 
@@ -262,17 +197,17 @@ Pcompile (char *pattern, idx_t size, reg_syntax_t ignored, bool exact)
   pcre2_set_compile_extra_options (ccontext, extra_options);
 #endif
 
-  void *re_storage = NULL;
+  void *re_storage = nullptr;
   if (match_lines)
     {
 #ifndef PCRE2_EXTRA_MATCH_LINE
-      static char const /* These sizes omit trailing NUL.  */
-        xprefix[4] = "^(?:", xsuffix[2] = ")$";
-      idx_t re_size = size + sizeof xprefix + sizeof xsuffix;
+      static char const *const xprefix = "^(?:";
+      static char const *const xsuffix = ")$";
+      idx_t re_size = size + strlen (xprefix) + strlen (xsuffix);
       char *re = re_storage = ximalloc (re_size);
-      char *rez = mempcpy (re, xprefix, sizeof xprefix);
+      char *rez = mempcpy (re, xprefix, strlen (xprefix));
       rez = mempcpy (rez, pattern, size);
-      memcpy (rez, xsuffix, sizeof xsuffix);
+      memcpy (rez, xsuffix, strlen (xsuffix));
       pattern = re;
       size = re_size;
 #endif
@@ -281,18 +216,20 @@ Pcompile (char *pattern, idx_t size, reg_syntax_t ignored, bool exact)
     {
       /* PCRE2_EXTRA_MATCH_WORD is incompatible with grep -w;
          do things the grep way.  */
-      static char const /* These sizes omit trailing NUL.  */
-        wprefix[10] = "(?<!\\w)(?:", wsuffix[7] = ")(?!\\w)";
-      idx_t re_size = size + sizeof wprefix + sizeof wsuffix;
+      static char const *const wprefix = "(?<!\\w)(?:";
+      static char const *const wsuffix = ")(?!\\w)";
+      idx_t re_size = size + strlen (wprefix) + strlen (wsuffix);
       char *re = re_storage = ximalloc (re_size);
-      char *rez = mempcpy (re, wprefix, sizeof wprefix);
+      char *rez = mempcpy (re, wprefix, strlen (wprefix));
       rez = mempcpy (rez, pattern, size);
-      memcpy (rez, wsuffix, sizeof wsuffix);
+      memcpy (rez, wsuffix, strlen (wsuffix));
       pattern = re;
       size = re_size;
     }
 
-  pcre2_set_character_tables (ccontext, pcre2_maketables (gcontext));
+  if (!localeinfo.multibyte)
+    pcre2_set_character_tables (ccontext, pcre2_maketables (gcontext));
+
   pc->cre = pcre2_compile ((PCRE2_SPTR) pattern, size, flags,
                            &ec, &e, ccontext);
   if (!pc->cre)
@@ -306,15 +243,15 @@ Pcompile (char *pattern, idx_t size, reg_syntax_t ignored, bool exact)
   free (re_storage);
   pcre2_compile_context_free (ccontext);
 
-  pc->mcontext = NULL;
+  pc->mcontext = nullptr;
   pc->data = pcre2_match_data_create_from_pattern (pc->cre, gcontext);
 
-  ec = pcre2_jit_compile (pc->cre, PCRE2_JIT_COMPLETE);
-  if (ec && ec != PCRE2_ERROR_JIT_BADOPTION && ec != PCRE2_ERROR_NOMEMORY)
-    die (EXIT_TROUBLE, 0, _("JIT internal error: %d"), ec);
+  /* Ignore any failure return from pcre2_jit_compile, as that merely
+     means JIT won't be used during matching.  */
+  pcre2_jit_compile (pc->cre, PCRE2_JIT_COMPLETE);
 
   /* The PCRE documentation says that a 32 KiB stack is the default.  */
-  pc->jit_stack = NULL;
+  pc->jit_stack = nullptr;
   pc->jit_stack_size = 32 << 10;
 
   pc->empty_match[false] = jit_exec (pc, "", 0, 0, PCRE2_NOTBOL);
@@ -379,7 +316,7 @@ Pexecute (void *vcp, char const *buf, idx_t size, idx_t *match_size,
 
           e = jit_exec (pc, subject, line_end - subject,
                         search_offset, options);
-          if (!bad_utf8_from_pcre2 (e))
+          if (MATCH_INVALID_UTF || !bad_utf8_from_pcre2 (e))
             break;
 
           idx_t valid_bytes = pcre2_get_startchar (pc->data);
